<?php
/**
* CedCommerce
*
* NOTICE OF LICENSE
*
* This source file is subject to the End User License Agreement (EULA)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* https://cedcommerce.com/license-agreement.txt
*
* @category    Ced
* @package     Ced_Delhivery
* @author      CedCommerce Core Team <connect@cedcommerce.com>
* @copyright   Copyright CedCommerce (https://cedcommerce.com/)
* @license      https://cedcommerce.com/license-agreement.txt
*/
namespace Ced\Delhivery\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;

class FetchPincode extends \Magento\Backend\App\Action
{
  /**
  * @var PageFactory
  */
  protected $resultPageFactory;
  /**
   * DB Storage table name
   */
  const TABLE_NAME = 'ced_delhivery_pincode';
  /**
   * Code of "Integrity constraint violation: 1062 Duplicate entry" error
   */
  const ERROR_CODE_DUPLICATE_ENTRY = 23000;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scope;
    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected $connection;
    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resource;
    /**
     * @var \Ced\Delhivery\Helper\Data
     */
    protected $helper;

    /**
  * @param Context $context
  * @param PageFactory $resultPageFactory
  */
  public function __construct(
    Context $context,
    PageFactory $resultPageFactory,
    \Magento\Framework\App\ResourceConnection $resource,
    \Ced\Delhivery\Helper\Data $helper,
    \Magento\Framework\App\Config\ScopeConfigInterface $scope
  ) {
    parent::__construct($context);
    $this->resultPageFactory = $resultPageFactory;
    $this->scope=$scope;
    $this->connection = $resource->getConnection();
    $this->resource = $resource;
    $this->helper = $helper;
  }

  /**
  * Index action
  *
  * @return void
  */
  public function execute()
  {
    $model = $this->_objectManager->create('Ced\Delhivery\Model\Pincode');
    $apiurl =  $this->scope->getValue('carriers/delhivery/pincode_url');
    $token = $this->scope->getValue('carriers/delhivery/license_key');
    if(count($model->getCollection()->getData())){
      $connection = $model->getCollection()->getConnection();
      $tableName = $model->getCollection()->getMainTable();
      $connection->truncateTable($tableName);
    }

    if($apiurl && $token){
      $path = $apiurl.'json/?token='.$token.'&pre-paid=Y';
      $retValue = $this->helper->Executecurl($path,'','');
      $codes = json_decode($retValue,true);

      /*if data is not proper then return*/
      if(is_null($codes) || !isset($codes['delivery_codes']) || !is_array($codes['delivery_codes'])){
        $this->messageManager->addErrorMessage(__('No Data Found, please try again later'));
        return $this->_redirect('delhivery/grid/index');
      }

      try {
        $data = [];
        $pincodes = [];
        foreach ($codes['delivery_codes'] as $item) {


          $zipcode = $item['postal_code']['pin'];
          if(!in_array($zipcode, $pincodes) && strlen($zipcode)==6){
            $data[] = ['district'=>$item['postal_code']['district'],
                      'pin'=>$zipcode,
                      'pre_paid'=>$item['postal_code']['pre_paid'],
                      'cash'=>$item['postal_code']['cash'],
                      'pickup'=>$item['postal_code']['pickup'],
                      'cod'=>$item['postal_code']['cod'],
                      'state_code'=>$item['postal_code']['state_code']
            ];
            $pincodes[] = $zipcode;
          }
        }
        if(count($data)){
          $tableName = $this->resource->getTableName(self::TABLE_NAME);
          $this->connection->insertMultiple($tableName, $data);
          $this->messageManager->addSuccessMessage(__('successfully uploaded'));
        }
      }catch (\Exception $e) {
        $errormsg = '';
        if ($e->getCode() === self::ERROR_CODE_DUPLICATE_ENTRY
                && preg_match('#SQLSTATE\[23000\]: [^:]+: 1062[^\d]#', $e->getMessage())
        ) {

              $errormsg =  __('URL key for specified store already exists.');
        }
        $this->messageManager->addErrorMessage(__($errormsg ? $errormsg : $e));
      }

    }else{
      $this->messageManager->addErrorMessage(__('Please Fill The Configuration First'));
    }
    return $this->_redirect('delhivery/grid/index');
  }

}
