<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Delhivery
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CedCommerce (https://cedcommerce.com/)
 * @license      https://cedcommerce.com/license-agreement.txt
 */
namespace  Ced\Delhivery\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\Response\Http\FileFactory;
use Magento\Backend\Model\View\Result\ForwardFactory;

class ShippingLabel extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */

    /**
     * @var FileFactory
     */
    protected $_fileFactory;

    /**
     * @var ForwardFactory
     */
    protected $resultForwardFactory;
    /**
     * @var \Ced\Delhivery\Model\AwbFactory
     */
    protected $awbFactory;
    /**
     * @var \Ced\Delhivery\Model\ShippingLabel
     */
    protected $shippingLabel;
    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $dateTime;
    /**
     * @var \Magento\Sales\Model\OrderFactory
     */
    protected $orderFactory;

    /**
     * @param Context $context
     * @param FileFactory $fileFactory
     * @param ForwardFactory $resultForwardFactory
     */
    public function __construct(
        Context $context,
        FileFactory $fileFactory,
        \Ced\Delhivery\Model\AwbFactory $awbFactory,
        \Ced\Delhivery\Model\ShippingLabel $shippingLabel,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $dateTime,
        ForwardFactory $resultForwardFactory
    ) {
        $this->_fileFactory = $fileFactory;
        $this->resultForwardFactory = $resultForwardFactory;
        $this->awbFactory = $awbFactory;
        $this->shippingLabel = $shippingLabel;
        $this->dateTime = $dateTime;
        $this->orderFactory = $orderFactory;
        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|\Magento\Backend\Model\View\Result\Forward
     */
    public function execute()
    {
    	$waybills=$this->getRequest()->getParam('selected',false);
    	$wayBills =[];
    	if(!$waybills){
    		$this->messageManager->addErrorMessage(__('There are no printable shipping labels related to selected waybills.'));
			$this->_redirect('*/grid/awb');
    	}
    	foreach ($waybills as $waybill) {
    	 	$awb = $this->awbFactory->create()->load(trim($waybill))->getData();

    	 	$wayBills[] = $awb['awb'];
    	}
		$flag = false;
		if (!empty($wayBills))
		{
			$labelperpage = 5;
			$totalpages = sizeof($wayBills)/$labelperpage;
	        $pdf = new \Zend_Pdf();
	        $style = new \Zend_Pdf_Style();
			for ($page_index = 0; $page_index <= $totalpages; $page_index++)
	        {
				$page = new \Zend_Pdf_Page(\Zend_Pdf_Page::SIZE_A4 );
				$pdf->pages[] = $page;
			}
			$pagecounter = 0;
			$i=0; $y=830;
			foreach ($wayBills as $_wayBills)
			{
				$orderid ='';
				$awb = $this->awbFactory->create()->load(trim($_wayBills),'awb')->getData();
				if(empty($awb))
				{
					continue;
				}

				$orderid = $awb['orderid'];
				$shipment_id = $awb['shipment_id'];

				$i++;
				if($i%$labelperpage == 0)
				{
					$pagecounter++; // Set to use new page
					$y = 830; // Set position for first label on new page
				}

				//$pdf->pages[$pagecounter];
				$order = $this->orderFactory->create()->load($orderid);
				if ($order->getData())
				{
					$flag = true;
					foreach ($order->getShipmentsCollection() as $shipment)
		  			{
		  				if(count($shipment->getData()) && $shipment->getEntityId()==$shipment_id){
		  					$this->shippingLabel->getContent($pdf->pages[$pagecounter], $shipment->getEntityId(), $_wayBills, $order,$y);
		  				}
		  			}
				}
				$y = $y-190;
			}

			if ($flag)
			{

				$date = $this->dateTime->date('Y-m-d_H-i-s');
		                return $this->_fileFactory->create(
		                    'shippinglabel'. $date . '.pdf',
		                    $pdf->render(),
		                    DirectoryList::VAR_DIR,
		                    'application/pdf'
		                );
			}
			else
			{
				$this->messageManager->addErrorMessage(__('There are no printable shipping labels related to selected waybills.'));
			}
		}
		$this->_redirect('*/grid/awb');

    }


}
