<?php
/**
 * CedCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End User License Agreement (EULA)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://cedcommerce.com/license-agreement.txt
 *
 * @category    Ced
 * @package     Ced_Delhivery
 * @author      CedCommerce Core Team <connect@cedcommerce.com>
 * @copyright   Copyright CedCommerce (https://cedcommerce.com/)
 * @license      https://cedcommerce.com/license-agreement.txt
 */
namespace Ced\Delhivery\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;

class UpdateAwb extends \Magento\Backend\App\Action
{
    /**
     * @var PageFactory
     */
    protected $resultPageFactory;
    /**
     * @var \Ced\Delhivery\Model\AwbFactory
     */
    protected $awbFactory;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scope;
    /**
     * @var \Ced\Delhivery\Helper\Data
     */
    protected $helper;

    /**
     * @param Context $context
     * @param PageFactory $resultPageFactory
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        \Ced\Delhivery\Model\AwbFactory $awbFactory,
        \Ced\Delhivery\Helper\Data $helper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scope 
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->awbFactory = $awbFactory;
        $this->scope=$scope;
        $this->helper = $helper;
    }
 
    /**
     * Index action
     *
     * @return void
     */
    public function execute()
    { 
      //$awbModel = $this->_objectManager->create('Ced\Delhivery\Model\Awb'); 
      $apiurl =  $this->scope->getValue('carriers/delhivery/gateway_url');
      $token = $this->scope->getValue('carriers/delhivery/license_key');
      if($apiurl && $token){
        $waybills = $this->awbFactory->create()->getCollection()->addFieldToFilter('state',1);
        
        if(count($waybills->getData()))
        { //No update to perform if count is zero
          $awbs = '';
          $i = 0;
          $y = 0;
          $awbStack = [];
          foreach($waybills as $waybill)
          {  
              if(isset($waybill['awb']) && $waybill['awb'])
                  $awbs .= $awbs ? ','.$waybill['awb'] : $waybill['awb']; 
              $i++;
              $y++;
              if($i>50 || $y==count($waybills)){
                $awbStack[] = $awbs;
                $awbs = '';
                $i = 0;
              }    
          }
          $errorMessage = '';
          $successCount = 0;
          foreach($awbStack as $awbStackData){
              $path = $apiurl.'api/v1/packages/json/?verbose=0&token='.$token.'&waybill='.$awbStackData; 
              $retValue = $this->helper->Executecurl($path,'','');
              $statusupdates = json_decode($retValue);
              
              if(empty($retValue))
              {
                $errorMessage = 'No update for AWB yet.';
              }elseif(!empty($retValue) && isset($statusupdates->Error)){
                $errorMessage = $statusupdates->Error;
              }else{
                
                foreach ($statusupdates->ShipmentData as $item) {
                   $model = $this->awbFactory->create()->load($item->Shipment->AWB,'awb');
                   $data = [];
                   $data['awb'] = $item->Shipment->AWB;  
                   $data['status'] = $item->Shipment->Status->Status;
                   $model->addData($data);
                   $model->save();
                   $successCount++;
                }
              }
          }
          if($successCount){
            $this->messageManager->addSuccessMessage(__($successCount.' Waybill(s) Updated Successfully'));
          }
          /*var_dump($y);
          var_dump($successCount);
          var_dump($errorMessage);die;*/
          if($errorMessage){
            $this->messageManager->addErrorMessage(__($y-$successCount.' Waybill(s) not updated due to '.$errorMessage));
          }  
      }else{
        $this->messageManager->addErrorMessage( __('No update for AWB yet.') );
      } 
    }
    else
    {
      $this->messageManager->addErrorMessage( __('Please add valid License Key and Gateway URL in plugin configuration') );
    }
    $this->_redirect('delhivery/grid/awb');
  }
}
